@@ -59,7 +59,7 @@ our %EXPORT_TAGS = (
 	time	=> [ qw(generalizedTime_to_time time_to_generalizedTime) ],
 );
 
-our $VERSION = '0.18';
+our $VERSION = '0.19';
 
 =item ldap_error_name ( ERR )
 
@@ -753,7 +753,7 @@ my $generalizedTime = shift;
 
     # Time::Local's timegm() interpret years strangely
     if ($year >= 1000) {
-      $dec = "0.$dec";
+      $dec = defined($dec) ? "0.$dec" : 0;
 
       # decimals in case of missing minutes / seconds - see RFC 4517
       if (!defined($min)) {
@@ -790,12 +790,15 @@ my $generalizedTime = shift;
 
           # make decimal part directional
           if ($dec != 0) {
+            my $sign = '';
+
             if ($time < 0) {
               $dec = 1 - $dec;
               $time++; 
+              $sign = '-'  if ($time == 0);
             }
             $dec =~ s/^0\.//;
-            $time .= ".$dec";
+            $time = "${sign}${time}.${dec}";
           }
 
           return $time;
@@ -841,19 +844,20 @@ sub time_to_generalizedTime($;@)
 my $arg = shift;
 my %opt = @_;
 
-  if ($arg =~ /^(\-?\d*)(?:[.,](\d*))?$/) {
-    my ($time, $dec) = ($1, $2);
+  if ($arg =~ /^(\-?)(\d*)(?:[.,](\d*))?$/) {
+    my ($sign, $time, $dec) = ($1, $2, $3);
 
     $dec = defined($dec) ? "0.$dec" : 0;
 
     # decimal part of time is directional: make sure to have it positive
-    if ($time < 0 && $dec != 0) {
-      $time--;
-      $dec = 1 - $dec;
+    if ($sign) {
+      if ($dec != 0) {
+        $time++;
+        $dec = 1 - $dec;
+      }
+      $time = -$time;
     }
 
-    $time = int($time);
-
     my ($sec,$min,$hour,$mday,$month,$year,$wday,$yday,$isdst) = gmtime(int($time));
 
     # generalizedTime requires 4-digit year without sign
